/*
 * decompose - split a firmware file
 *
 * Copyright (C) 2004 Robert Bowler
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * decompose - split a firmware file
 *
 * Copyright (C) 2004 Robert Bowler
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* I have not given a seconds thought to how this will work on big endian machines */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "utils.h"


int reverse_endian = 1;

void usage(void)
{
 fprintf(stderr, "usage :\n");
 fprintf(stderr, "decompose <firmware file>\n");
 exit(1);
}

void write_component(char *buf,
		     int len,
		     const char *filename)
{
  
 if (len > 0)
   {
     FILE *f = fopen(filename, "w");
     fwrite(buf, len, 1, f);
     fclose(f);
   }
}

void
verify_checksum(const char *description,
		unsigned checksum,
		void *data,
		int length)
{
  unsigned calculated_checksum = htocl(~crc32(data, length));
  printf("%s checksum = %u, calculated = %u\n", description,
	 checksum, calculated_checksum);
  
  if (checksum != calculated_checksum)
   {
     fprintf(stderr, "%s checksum failed\n", description);
     exit(1);
   }
}


int
main(int argc,
	char *argv[])
{
FILE *bin_file;
struct stat stat_buf;
char header_buf[256], *data_buf = NULL;
struct header_type head;
if (sizeof(head) != 256)
	{
	fprintf(stderr, "extra padding in header %d\n", sizeof(head));
	return 0;
	}
if (argc != 2)
	{
	usage();
	}
bin_file = fopen(argv[1], "r");
if(NULL == bin_file)
	{
	fprintf(stderr, "failed to open %s\n", argv[1]);
	return 1;
	}
stat(argv[1], &stat_buf);
if (stat_buf.st_size < 256)
	{
	fprintf(stderr, "not big enough to contain header\n");
	return 1;
	}
fread(header_buf, 256, 1, bin_file);
memcpy(&head, header_buf, sizeof(head));

printf("vendor=%.20s\n", head.vendor);
printf("version=%.20s\n", head.version);
printf("model=%.16s\n", head.model);
printf("image size=%.10s\n", head.image_size);
printf("loader address=%.12s\n", head.loader_addr);
printf("loader size=%.10s\n",head.loader_size);
printf("root fs address=%.12s\n", head.root_fs_addr);
printf("root fs size=%.10s\n", head.root_fs_size);
printf("kernel address=%.12s\n", head.kernel_addr);
printf("kernel size=%.10s\n", head.kernel_size);

/* validate */
if (256 + atoi(head.image_size) != stat_buf.st_size)
  {
    fprintf(stderr, "wrong file size in header\n");
    return 1;
  }
 verify_checksum("header", head.header_checksum, header_buf, 236);
 
 data_buf = malloc(atoi(head.image_size));
 fread(data_buf, atoi(head.image_size), 1, bin_file);
 fclose(bin_file);

 verify_checksum("data", head.data_checksum, data_buf, atoi(head.image_size));
 if (head.header_version == '2')
   {
     printf("header version = %c\n", head.header_version);
     verify_checksum("payload", head.payload_checksum,
		     data_buf + atoi(head.loader_size),
		     atoi(head.kernel_size) + atoi(head.root_fs_size));
   }
      
 write_component(header_buf, 256, "header");
 write_component(data_buf, atoi(head.loader_size), "loader");
 write_component(data_buf + atoi(head.loader_size), atoi(head.root_fs_size), "root_fs");
 write_component(data_buf + atoi(head.loader_size) + atoi(head.root_fs_size), atoi(head.kernel_size) ,"kernel");

 return 0;
}
